@@ -258,8 +258,8 @@ void TableDialog::on_clipboardButton_clicked() {
 
         updateSeparator(text);
 
-        // write file data to the temporary file
-        _tempFile->write(text.toLatin1());
+        // write file data to the temporary file in UTF-8
+        _tempFile->write(text.toUtf8());
 
         // close the file so the CSV reader can access it
         _tempFile->close();
