@@ -1,5 +1,6 @@
 #include "calendaritem.h"
 
+#include <qregularexpression.h>
 #include <services/owncloudservice.h>
 #include <utils/misc.h>
 
@@ -7,13 +8,14 @@
 #include <QDebug>
 #include <QMessageBox>
 #include <QRegularExpression>
-#include <QSettings>
 #include <QSqlDatabase>
 #include <QSqlError>
 #include <QSqlQuery>
 #include <QSqlRecord>
 #include <QUuid>
 
+#include "services/settingsservice.h"
+
 CalendarItem::CalendarItem() {
     id = 0;
     hasDirtyData = false;
@@ -41,6 +43,8 @@ QString CalendarItem::getRelatedUid() const { return this->relatedUid; }
 
 QString CalendarItem::getDescription() { return this->description; }
 
+QString CalendarItem::getTags() { return this->tags; }
+
 int CalendarItem::getPriority() { return this->priority; }
 
 bool CalendarItem::getHasDirtyData() { return this->hasDirtyData; }
@@ -79,6 +83,8 @@ void CalendarItem::setPriority(int value) { this->priority = value; }
 
 void CalendarItem::setCompleted(bool value) { this->completed = value; }
 
+void CalendarItem::setTags(QString tags) { this->tags = tags; };
+
 void CalendarItem::updateCompleted(bool value) {
     this->completed = value;
     if (value) {
@@ -253,7 +259,7 @@ bool CalendarItem::fillFromQuery(const QSqlQuery &query) {
     this->modified = query.value(QStringLiteral("modified")).toDateTime();
     this->completedDate = query.value(QStringLiteral("completed_date")).toDateTime();
     this->sortPriority = query.value(QStringLiteral("sort_priority")).toInt();
-
+    this->tags = query.value(QStringLiteral("tags")).toString();
     return true;
 }
 
@@ -472,6 +478,7 @@ bool CalendarItem::store() {
         query.prepare(
             "UPDATE calendarItem SET summary = :summary, url = :url, "
             "description = :description, "
+            "tags = :tags, "
             "has_dirty_data = :has_dirty_data, "
             "completed = :completed, "
             "calendar = :calendar, uid = :uid, related_uid = :related_uid, "
@@ -485,11 +492,11 @@ bool CalendarItem::store() {
     } else {
         query.prepare(
             "INSERT INTO calendarItem "
-            "(summary, url, description, calendar, uid, related_uid, ics_data, "
+            "(summary, url, description, tags, calendar, uid, related_uid, ics_data, "
             "etag, last_modified_string, has_dirty_data, completed, "
             "alarm_date, priority, created, modified, "
             "completed_date, sort_priority) "
-            "VALUES (:summary, :url, :description, :calendar, :uid,"
+            "VALUES (:summary, :url, :description, :tags, :calendar, :uid,"
             " :related_uid, :ics_data, :etag, :last_modified_string, "
             ":has_dirty_data, :completed, :alarm_date, :priority, "
             ":created, :modified, :completed_date, "
@@ -513,6 +520,7 @@ bool CalendarItem::store() {
     query.bindValue(QStringLiteral(":modified"), this->modified);
     query.bindValue(QStringLiteral(":completed_date"), this->completedDate);
     query.bindValue(QStringLiteral(":sort_priority"), this->sortPriority);
+    query.bindValue(QStringLiteral(":tags"), this->tags);
 
     // on error
     if (!query.exec()) {
@@ -543,6 +551,15 @@ QString CalendarItem::generateNewICSData() {
     // update the icsDataHash
     icsDataHash[QStringLiteral("SUMMARY")] = summary;
     icsDataHash[QStringLiteral("DESCRIPTION")] = description;
+    // If categories are empty, remove the CATEGORIES data
+    // otherwise we get empty tag
+    if (tags.isEmpty()) {
+        icsDataHash.remove(QStringLiteral("CATEGORIES"));
+        icsDataKeyList.removeAll(QStringLiteral("CATEGORIES"));
+    } else {
+        // Turn single backslashes into two for categories when uploading
+        icsDataHash[QStringLiteral("CATEGORIES")] = tags;
+    }
     icsDataHash[QStringLiteral("UID")] = uid;
     icsDataHash[QStringLiteral("PRIORITY")] = QString::number(priority);
     icsDataHash[QStringLiteral("PERCENT-COMPLETE")] = QString::number(completed ? 100 : 0);
@@ -601,8 +618,10 @@ QString CalendarItem::generateNewICSData() {
 
         QString line = icsDataHash.value(key);
 
-        // escape "\"s
-        line.replace(QLatin1String("\\"), QLatin1String("\\\\"));
+        // commas only have one backslash
+        if (key != QStringLiteral("CATEGORIES")) {
+            line.replace(QLatin1String("\\"), QLatin1String("\\\\"));
+        }
         // convert newlines
         line.replace(QLatin1String("\n"), QLatin1String("\\n"));
 
@@ -725,6 +744,12 @@ bool CalendarItem::updateWithICSData(const QString &icsData) {
     description = icsDataHash.contains(QStringLiteral("DESCRIPTION"))
                       ? icsDataHash[QStringLiteral("DESCRIPTION")]
                       : QString();
+
+    // Turn double backslashes into one for categories
+    tags = icsDataHash.contains(QStringLiteral("CATEGORIES"))
+               ? icsDataHash[QStringLiteral("CATEGORIES")]
+               : QString();
+
     priority = icsDataHash.contains(QStringLiteral("PRIORITY"))
                    ? icsDataHash[QStringLiteral("PRIORITY")].toInt()
                    : 0;
@@ -954,7 +979,13 @@ void CalendarItem::generateICSDataHash() {
             lastKey = findFreeHashKey(&icsDataHash, lastKey);
 
             // add new key / value pair to the hash
-            icsDataHash[lastKey] = decodeICSDataLine(match.captured(2));
+            // Categories/tags wont be cleaned since they can have commas in middle of
+            // a category
+            if (lastKey == "CATEGORIES") {
+                icsDataHash[lastKey] = match.captured(2);
+            } else {
+                icsDataHash[lastKey] = decodeICSDataLine(match.captured(2));
+            }
             //            hash.insert( lastKey, decodeICSDataLine(
             //            match.captured(2) ) );
 
@@ -1062,7 +1093,7 @@ CalendarItem CalendarItem::createNewTodoItem(const QString &summary, const QStri
  * @return
  */
 int CalendarItem::getCurrentCalendarIndex() {
-    QSettings settings;
+    SettingsService settings;
 
     QString todoListSelectorSelectedItem =
         settings.value(QStringLiteral("TodoDialog/todoListSelectorSelectedItem")).toString();
@@ -1087,7 +1118,7 @@ int CalendarItem::getCurrentCalendarIndex() {
  * @return
  */
 QString CalendarItem::getCurrentCalendarUrl() {
-    QSettings settings;
+    SettingsService settings;
 
     int index = getCurrentCalendarIndex();
 
